/*****************************************************************************
 *                                                                           *
 *                Licensed Materials - Property of IBM Corp.                 *
 *                  5648-C60 (c) Copyright IBM Corp. 2000                    *
 *                                                                           *
 *       MQJMS Standalone Application Server Facilities (ASF) Samples        *
 *                                                                           *
 *                                                                           *
 *  FILENAME:     LoggingMessageListenerFactory.java                         *
 *                                                                           *
 *  DESCRIPTION:  Defines both a 'LoggingMessageListener' which implements   *
 *                javax.jms.MessageListener and its factory class, used to   *
 *                create instances of it.                                    *
 *                                                                           *
 *                Further information about these classes and their usage    *
 *                can be found in the 'Websphere MQ Using Java'              *
 *                documentation.                                             *
 *                                                                           *
 *  USAGE:        n/a                                                        *
 *                                                                           *
 *  HISTORY:      010131 - First released as part of SupportPac MA88 v2      *
 *                                                                           *
 *****************************************************************************/

import javax.jms.*;
import java.util.*;
import java.io.*;

public class LoggingMessageListenerFactory implements MessageListenerFactory {

   private int number = 0;  // The number of listeners created by this factory
   private String name;                // An identifying name for this factory
   private BufferedWriter log;          // The log handle for listeners to use

   private Vector listeners = new Vector(); // References to all the listeners
                                                    // created by this factory

   /**************************************************************************
    *  Constructor - Pass in the name of this factory, and the name of the   *
    *                file to use as logging output                           *
    **************************************************************************/
   LoggingMessageListenerFactory( String name, String logFilename )
      throws IOException {

      this.name = name;
      this.log = new BufferedWriter( new FileWriter(logFilename) );
   }

   /**************************************************************************
    *  getName - Returns the name of this message listener fatcory           *
    **************************************************************************/
   public String getName() {
      return name;
   }

   /**************************************************************************
    *  createMessageListener - Creates and returns new instance of the       *
    *                          CountingMessageListener class (also defined   *
    *                          in this file: see below), giving it a name    *
    *                          made up of the name of this factory and the   *
    *                          number of listeners already created.          *
    **************************************************************************/
   public MessageListener createMessageListener() {
      String listenerName;
      int    i;

      // Create the listener name
      synchronized(this) {
         i = number;
         number++;
         listenerName = name+"-"+number;
      }

      // Create the listener based on the name we created above, and add a
      // reference to it to this factory's vector of created listeners,
      // ensuring we have the required capacity
      LoggingMessageListener lml = new LoggingMessageListener(listenerName,
                                                              log);
      synchronized(listeners) {
         listeners.setSize(Math.max(listeners.size(),i+1));
      }
      listeners.setElementAt(lml,i);

      // Finally return the listener
      return(lml);
   }
}

/*****************************************************************************
 *  LoggingMessageListener - A simple implementation of the MessageListener  *
 *                           interface which keeps track of the number of    *
 *                           messages it has been passed, but does nothing   *
 *                           with the message itself.                        *
 *****************************************************************************/
class LoggingMessageListener implements MessageListener {
   String name;                          // This listener's identifying string
   BufferedWriter log;                          // Log file handle for logging
   int count=0;        // A count of the number of messages it has been passed

   /** Constructor - Saves this message listener's identity and a log file
                     handle for future use */
   LoggingMessageListener( String name, BufferedWriter log ) {
      this.name = name;
      this.log  = log;
   }

   /** onMessage - Implements the javax.jms.MessageListener interface. This
                   implementation writes a log entry to this listener's log
                   file handle. The entry contains a timestamp, this listener's
                   identity, a message count and the contents of the message's
                   'value' property. */
   public void onMessage( Message m ) {
      // Increment counter
      count++;

      // Create a buffer to write the log entry to
      StringBuffer buf = new StringBuffer();

      // Write the log entry to the buffer...
      buf.append(beautifyDate(new GregorianCalendar()));
      buf.append(" "+name+" Msg#"+count);
      try {
         buf.append(" value="+m.getIntProperty("value"));
      } catch( JMSException jmse ) {
         System.out.println("Unable to obtain user property 'value': "+jmse);
      }
      buf.append("\n");

      // ...then write the entry to the log, if we can
      try {
         log.write(buf.toString());
         log.flush();
      } catch( IOException iox ) {
         System.out.println("Message listener unable to log message: "+iox );
      }
   }

   /** getName - Returns this message listener's identity */
   public String getName() { return name; }

   /** getCount - Returns the count of messages this listener has seen */
   public int getCount() { return count; }

   /** Private utility method to format a date nicely */
   private static StringBuffer beautifyDate( Calendar c ) {
      StringBuffer buf = new StringBuffer(23);

      buf.append(c.get(Calendar.YEAR)+"-");
      buf.append((c.get(Calendar.MONTH)+1)+"-");
      buf.append(c.get(Calendar.DATE)+" ");
      buf.append(c.get(Calendar.HOUR_OF_DAY)+":");
      buf.append(c.get(Calendar.MINUTE)+":");
      buf.append(c.get(Calendar.SECOND)+".");
      buf.append(c.get(Calendar.MILLISECOND));

      return(buf);
   }
}
